<?PHP
/**
* Library of functions and constants for module AutoView
*
* @package mod-autoview
* @category mod
* @author Tim Williams (EuroMotor/AutoTrain)
* @licence This code is copyright to EuroMotor AutoTrain LLP and is licenced under the GNU GPL Licence
*/

/**
* Fixes an upper limit for the user defined name of the Autoview instance
*/
define("AUTOVIEW_MAX_NAME_LENGTH", 50);

/**
* Given an object containing all the necessary data, 
* (defined by the form in mod.html) this function 
* will create a new instance and return the id number 
* of the new instance.
* @param object $autoview the form with all autoview parameters in
* @return the newly created instance id or false
*/
function autoview_add_instance($autoview) {

    $mainrecord->name = strip_tags($autoview->content);
    if (strlen($mainrecord->name) > AUTOVIEW_MAX_NAME_LENGTH) {
        $mainrecord->name = substr($mainrecord->name, 0, AUTOVIEW_MAX_NAME_LENGTH).'...';
    } else {
        if (strlen($mainrecord->name) == 0) {
            $mainrecord->name = "AutoView ".time();
        }
    }

    $mainrecord->timemodified = time();
    $mainrecord->course       = $autoview->course;
    $mainrecord->content      = $autoview->content;
    if ($autoview->noframe == '1'){
        $mainrecord->noframe  = 1;
    } else {
        $mainrecord->noframe  = 0; 
    }
    if (isset($autoview->configfile) && strlen($autoview->configfile) > 0){
        $mainrecord->configfile = $autoview->configfile;
    } else {
        $mainrecord->configfile = str_replace(' ', '_', trim($mainrecord->name)).'.avx';
    }
    $instance_id = insert_record('autoview', $mainrecord);
    autoview_check_configfile($mainrecord->configfile, $autoview->course);
    return $instance_id;
}


/**
* Given an object containing all the necessary data, 
* (defined by the form in mod.html) this function 
* will update an existing instance with new data.
* @param object $autoview the form with all autoview parameters in
* @return the boolean status telling it succeeded
*/
function autoview_update_instance($autoview) {

    $mainrecord->name = strip_tags($autoview->content);
    if (strlen($mainrecord->name) > AUTOVIEW_MAX_NAME_LENGTH) {
        $mainrecord->name = substr($mainrecord->name, 0, AUTOVIEW_MAX_NAME_LENGTH).'...';
    }

    $mainrecord->id           = $autoview->instance;
    $mainrecord->timemodified = time();
    $mainrecord->course       = $autoview->course;
    $mainrecord->content      = $autoview->content;
    $mainrecord->configfile   = $autoview->configfile;
    if ($autoview->noframe == '1'){
        $mainrecord->noframe  = 1;
    } else {
        $mainrecord->noframe  = 0;
    }    

    $ret = update_record('autoview', $mainrecord);
    autoview_check_configfile($autoview->configfile, $autoview->course);
    return $ret;
}

/**
* Checks existance of the configuration file and creates a new file if using a default
* @param string $file the location of the config file relative to the current course file container
* @param int $courseid the current course id
* @uses CFG
* @return void
*/
function autoview_check_configfile($file, $courseid) {
    global $CFG;

    $fileloc = $CFG->dataroot.'/'.$courseid;
    // Check the directory exists and create if necessary
    if (!file_exists($fileloc)){
        mkdir($fileloc, $CFG->directorypermissions);
    }
    
    $fileloc = $fileloc.'/'.$file;
    if (!file_exists($fileloc))
        copy($CFG->dirroot.'/mod/autoview/avedit/blank.avx', $fileloc);
}

/**
* Given an ID of an instance of this module, 
* this function will permanently delete the instance 
* and any data that depends on it.  
* @param int the instance to delete
* @return true if successfully deleted
*/
function autoview_delete_instance($id) {

    if (! $autoview = get_record('autoview', 'id', $id)) {
        return false;
    }

    $result = true;

    if (!delete_records('autoview', 'id', $autoview->id)) {
        $result = false;
    }

    return $result;
}

/**
* Returns the users with data in one resource
* (NONE, but must exist on EVERY mod !!)
* @param int $autoviewid
*/
function autoview_get_participants($autoviewid) {
    return false;
}

/**
* Given a course_module object, this function returns any 
* "extra" information that may be needed when printing
* this activity in a course listing.
*
* See get_array_of_activities() in course/lib.php
* @param object $coursemodule the instance of the course module for this autoview
*/
function autoview_get_coursemodule_info($coursemodule) {
   return false;
}

/**
* realizes a complete application output from the autoview client side implementation. 
* This function is PHP5/PHP4 compliant.
* @param string $datafile the content of a configuration file
* @param string $stylesheetfile an XSL transformer for injectong config data and making application
* @param array $parameters a set of parameters that can be passed from Moodle context
* @return the produced html, depending on application context
*/
function process_xsl($datafile, $stylesheetfile, $parameters) {
    
    if (class_exists('XSLTProcessor')){

/// PHP 5 Style XSL processor

        $doc = new DOMDocument();
        $xsltproc = new XSLTProcessor();
        $doc->load($stylesheetfile);
        $xsltproc->importStyleSheet($doc);
        $doc->load($datafile);

        foreach ($parameters as $key => $value){
            $xsltproc->setParameter('', $key, $value);
        }
        return $xsltproc->transformToXML($doc);

    } elseif (function_exists('xslt_create')) {

/// PHP 4 Sablot Style XSL processor
    
        $xsltproc = xslt_create();
        
        $html = xslt_process($xsltproc, $datafile, $stylesheetfile, NULL, NULL, $parameters);
        
        if (empty($html)){
            die('XSLT processing error: '. xslt_error($xsltproc));
        }
        xslt_free($xsltproc);
        return $html;
    
    } elseif (function_exists('domxml_xslt_stylesheet_file')) {

/// PHP 4 DOM Style XSL processor

        $datadoc = domxml_open_file($datafile);
        $stylesheet = domxml_xslt_stylesheet_file($stylesheetfile);
        $htmldoc = $stylesheet->process($datadoc, $parameters);
        return $htmldoc->html_dump_mem();

    } else {
        return '<p>'.get_string('noxsl', 'autoview').'</p>';
    }
}

/**
* Tests availability of the XSLT processor
* @return true if exists
*/
function testXSL(){
    if (method_exists('XSLTProcessor', 'setParameter') || 
           function_exists('domxml_xslt_stylesheet_file(') ||
                function_exists('xslt_create')) {
       return true;
    }

    return false;
}

/**
* Tests availability of the Pecl extension
* @return true if exists
*/
function testPeclHttp(){
    if (function_exists('http_post_fields') && function_exists('http_parse_message')){
        return true;
    } else {
        return false;
    }
}
?>